<?php

/**
 * Erpal Project Time Tracking Report
 *
 * creates project management report with the following options:
 *
 * projects => list of projects
 * no_cache => ignore caching
 *
 * @author Yann Karl
 */
class ErpalProjectReportsManagementReport {
  /**
   * create data for project management report, returns false if no
   * data available
   *
   * @param array $options
   * @return mixed
   */
  static function create($options = array()) {
    $taxonomy_idlist = self::taskStatusTaxonomyIdlistCreate();

    $options = array_merge(self::defaults(), $options);

    $retval = array('open' => 0, 'overdue' => 0, 'not_assigned' => 0, 'no_time_estimate' => 0);

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')->entityCondition('bundle', 'erpal_task');
    $query->fieldCondition('field_task_status_term', 'tid', $taxonomy_idlist['in_progress'], 'IN');
    $query->addMetaData('account', user_load(1)); // Run the query as user 1.

    if (count($options ['projects']) > 0) {
      $query->fieldCondition('field_project_ref', 'target_id', $options ['projects'], 'IN');
    }

    $retval['open'] = (int)($query->count()->execute());

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')->entityCondition('bundle', 'erpal_task');
    $query->fieldCondition('field_task_status_term', 'tid', $taxonomy_idlist['in_progress'], 'IN');
    $query->fieldCondition('field_date', 'value', sprintf('%s, 23:59:59', date(DATE_FORMAT_DATE, time())), '<=');
    $query->addMetaData('account', user_load(1)); // Run the query as user 1.

    if (count($options ['projects']) > 0) {
      $query->fieldCondition('field_project_ref', 'target_id', $options ['projects'], 'IN');
    }

    $retval['overdue'] = (int)($query->count()->execute());

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')->entityCondition('bundle', 'erpal_task');
    $query->addTag('filter_task_no_assigned_users');
    $query->addMetaData('account', user_load(1)); // Run the query as user 1.

    if (count($options ['projects']) > 0) {
      $query->addTag('filter_projects');
      $query->addMetaData('projects', $options ['projects']);
    }

    $retval['not_assigned'] = (int)($query->count()->execute());

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')->entityCondition('bundle', 'erpal_task');
    $query->addTag('filter_task_no_estimated_time');
    $query->addMetaData('account', user_load(1)); // Run the query as user 1.
    
    if (count($options ['projects']) > 0) {
      $query->addTag('filter_projects');
      $query->addMetaData('projects', $options ['projects']);
    }

    $retval['no_time_estimate'] = (int)($query->count()->execute());

    return $retval;
  }

  /**
   * get default options for project management report creation
   *
   * @return array
   */
  static function defaults() {
    return array (
      'projects' => array(),
      'no_cache' => FALSE
    );
  }

  /**
   * load cached erpal report object for project management report from database
   * if none available or dirty, new object is created and stored
   *
   * @param array $options
   * @return array
   */
  static function load($options = array()) {
    $options = array_merge(self::defaults(), $options);

    // try to load entity with given options from db, check if dirty and return cached data if not
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'erpal_reports')
    ->entityCondition('bundle', 'erpal_project_management_reports');

    if (count($options['projects']) > 0) {
      $query->fieldCondition('field_project_ref', 'target_id', $options['projects'], 'IN');
    }

    $result = $query->execute();

    $entity = NULL;
    if (count($result) > 0) {
      $entity = entity_load_single('erpal_reports', key($result ['erpal_reports']));
      $entity_wrapper = entity_metadata_wrapper('erpal_reports', $entity);

      if ($options['no_cache'] == FALSE && $entity_wrapper->dirty->value() == 0) {
        return drupal_json_decode($entity_wrapper->data->value());
      }
    }

    $retval = self::create($options);

    if ($entity == NULL) {
      $entity = entity_create('erpal_reports', array('type' =>'erpal_project_management_reports'));
    }

    // store data to entity, reset dirty flag
    $entity->data = drupal_json_encode($retval);
    $entity->dirty = 0;

    if (count($options['projects']) > 0) {
      $projects = array();
      foreach($options['projects'] as $project) {
        $projects[] = array('target_id' => $project);
      }
      $entity->field_project_ref = array(LANGUAGE_NONE => $projects);
    }

    $entity->save();

    return $retval;
  }

  /**
   * create list of task status tids from task status term taxonomy
   *
   * @param array $tid_aborted
   * @param array $tid_in_progress
   * @param array $tid_completed
   */
  static function taskStatusTaxonomyIdlistCreate() {
    $taxonomy = taxonomy_vocabulary_machine_name_load('task_status_terms');

    foreach(taxonomy_get_tree($taxonomy->vid) as $term) {
      $term_dataset = taxonomy_term_load($term->tid);
      $term_wrapper = entity_metadata_wrapper('taxonomy_term', $term_dataset);
      $process_status = $term_wrapper->field_simple_process_status->value();

      switch($process_status) {
        case 'aborted':
          $tid_aborted[] = $term_wrapper->tid->value();
          break;
        case 'in_progress':
          $tid_in_progress[] = $term_wrapper->tid->value();
          break;
        case 'completed':
          $tid_completed[] = $term_wrapper->tid->value();
          break;
      }
    }

    return array('aborted' => $tid_aborted, 'in_progress' => $tid_in_progress, 'completed' => $tid_completed);
  }
}

