<?php

/**
 * Erpal Project Time Tracking Report
 *
 * creates a scaled time tracking report with the following options:
 *
 * projects => list of projects
 * customers => list of customers
 * customer_categories => list of customer categories
 * service_categories => list of service categories
 * users => list of users
 * dim_project => add project dimension
 * dim_customer => add customer dimension
 * dim_customercategory => add customer category dimension
 * dim_servicecategory => add service category dimension
 * dim_user => add user dimension
 * no_cache => ignore caching
 *
 * @author Manuel Pistner
 */
class ErpalProjectReportsTimeTracking {
  /**
   * create data for project report, returns false if no
   * data available
   *
   * @param array $options
   * @return mixed
   */
  static function create($options = array()) {
    $options = array_merge(self::defaults(), $options);

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', 'erpal_task')
      ->addMetaData('account', user_load(1)); // Run the query as user 1.

    $noresult = FALSE;

    $has_project_subquery = FALSE;
    $subquery_projects = array();

    if (!empty($options['status']) || !empty($options['profit_status'])) {  //filter in projects with the selected status
      $query_project = new EntityFieldQuery();
      $query_project->entityCondition('entity_type', 'node')->entityCondition('bundle', 'erpal_project');      
      $query_project->addMetaData('account', user_load(1)); // Run the query as user 1.
      
      if (!empty($options['status'])) {
        $query_project->fieldCondition('field_project_status_term', 'tid', $options['status'], 'IN');
      }
      
      $result = $query_project->execute();

      //check if we want to group by profitable or none profitable projects
      $project_nids = array();
      if (!empty($options['profit_status'])) {
        foreach ($result['node'] as $project_nid=>$query_data) {
          $margin_options = array('projects' => array($project_nid), 'no_cache' => FALSE);
          
          $profit_margin_report = ErpalProjectReportsProfitMargin::load($margin_options, true);
          
          if (!$profit_margin_report || empty($profit_margin_report['details']))
            continue;

          $profit_status = $options['profit_status'];          
          $profit_treshold = intval($options['profit_treshold']);
          $profit_arr = (array)$profit_margin_report['details']; //profit in percent
          $profit_arr = reset($profit_arr);

          $profit = $profit_arr->profit->profit_percent; 

          if ($profit_status == 'profitable') {
            if ($profit >= $profit_treshold) {              
              $project_nids[$project_nid] = $project_nid;
            }
          } elseif ($profit_status ==  'non_profitable') {            
            if ($profit < $profit_treshold) {              
              $project_nids[$project_nid] = $project_nid;
            }
          }
        }
      }
          
      if (count($project_nids) > 0) {
        $has_project_subquery = TRUE;
        $subquery_projects = $project_nids;
      } else {
        $noresult = TRUE;
      }
    }

    //set the date range for the timetracking bookings
    $date_from = !empty($options['date_from']) ? $options['date_from'] : false;
    $date_till = !empty($options['date_till']) ? $options['date_till'] : false;
    
    // categories: field_project_tags
    if (count($options['task_categories']) > 0) {
      $query->fieldCondition('field_project_tags', 'tid', $options ['task_categories'], 'IN');
    }

    // project: field_project_ref
    if (count($options['projects']) > 0) {
      $subquery_projects = count($subquery_projects) == 0 ? $options ['projects'] : array_intersect($subquery_projects, $options ['projects']);
    }

    if (count($subquery_projects)) {
      $query->fieldCondition('field_project_ref', 'target_id', $subquery_projects, 'IN');
    }

    $result = $query->execute();
    $data = array();
    $details = array();
    if (!isset($result ['node']) || count($result ['node']) == 0) {
      $noresult = TRUE;
    }

    if ($noresult === FALSE) {
      $nids = array_keys($result ['node']);

      foreach($nids as $nid) {
        $node = node_load($nid);
        $node_wrapper = entity_metadata_wrapper('node', $node);
        
        $tag = $node_wrapper->field_project_tags->value();

        if ($tag != NULL) {
          $tag_name = $tag->name;
        } else {
          $tag_name = t('No Category');
        }

        $field_date = $node_wrapper->field_date->value();
        $field_project = $node_wrapper->field_project_ref->value();

        $series_estimated_title = t("Estimated");
        $series_duration_title = t("Duration");
        $series_addon = array();

        if ($options['dim_project'] == TRUE) {
          $series_addon[] = $field_project->title;
        }

        if (count($series_addon) > 0){
          $series_estimated_title .= sprintf(' - %s', implode(', ', $series_addon));
          $series_duration_title .= sprintf(' - %s', implode(', ', $series_addon));
        }

        if (empty($data[$series_estimated_title])) {
          $data[$series_estimated_title] = array();
          $details[$series_estimated_title] = array ();
        }

        if (empty($data[$series_estimated_title][$tag_name])) {
          $data[$series_estimated_title][$tag_name] = 0;
          $details[$series_estimated_title][$tag_name] = array();
        }

        if (empty($data[$series_duration_title])) {
          $data[$series_duration_title] = array();
          $details[$series_duration_title] = array();
        }

        if (empty($data[$series_duration_title][$tag_name])) {
          $data[$series_duration_title][$tag_name] = 0;
          $details[$series_duration_title][$tag_name] = array();
        }

        $time_information = _erpal_projects_helper_direct_time_information($node->nid, false, $date_from, $date_till);        
        
        $estimated_time = $time_information['estimation'];
        $total_time = $time_information['summed_time'];
        
        $data[$series_estimated_title][$tag_name] += $estimated_time;
        $data[$series_duration_title][$tag_name] += $total_time;
        
        $details[$series_duration_title][$tag_name][] = array('entity_type' => 'node', 'id' => $node->nid, 'title' => $node->title, 'date' => $tag_name, 'value' => $total_time);
        
        $details[$series_estimated_title][$tag_name][] = array('entity_type' => 'node', 'id' => $node->nid, 'title' => $node->title, 'date' => $tag_name, 'value' => $estimated_time);
      }
    } else {
      $data = FALSE;
    }

    if (is_array($data) && count($data) == 0) {
      $data = false;
    }

    //build extended report data
    return array(
      'data' => $data,
      'extended' => array(), //Not yet needed
      'details' => $details,
    );
  }

  /**
   * get default options for project time tracking report creation
   *
   * @return array
   */
  static function defaults() {
    return array (
        'task_categories' => array(),
        'projects' => array(),
        'customers' => array (),
        'customer_categories' => array (),
        'users' => array(),
        'dim_project' => FALSE,
        'dim_customer' => FALSE,
        'dim_customercategory' => FALSE,
        'dim_user' => FALSE,
        'no_cache' => FALSE
    );
  }

  /**
   * load cached erpal report object for project time tracking report from database
   * if none available or dirty, new object is created and stored
   *
   * @param array $options
   * @return array
   */
  static function load($options = array()) {
    $options = array_merge(self::defaults(), $options);

    // try to load entity with given options from db, check if dirty and return cached data if not
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'erpal_reports')
    ->entityCondition('bundle', 'erpal_project_timetracking_reports')
    ->fieldCondition('field_erp_prj_tt_rep_dim_prj', 'value', $options['dim_project'] === TRUE ? 1 : 0, '=')
    ->fieldCondition('field_erp_prj_tt_rep_dim_cust', 'value', $options['dim_customer'] === TRUE ? 1 : 0, '=')
    ->fieldCondition('field_erp_prj_tt_rep_dim_cuscat', 'value', $options['dim_customercategory'] === TRUE ? 1 : 0, '=')
    ->fieldCondition('field_erp_prj_tt_rep_dim_user', 'value', $options['dim_user'] === TRUE ? 1 : 0, '=');

    if (count($options['customers']) > 0) {
      $query->fieldCondition('field_customer_ref', 'target_id', $options['customers'], 'IN');
    }
    if (count($options['customer_categories']) > 0) {
      $query->fieldCondition('field_customer_category', 'value', $options['customer_categories'], 'IN');
    }
    if (count($options['projects']) > 0) {
      $query->fieldCondition('field_project_ref', 'target_id', $options['projects'], 'IN');
    }

    $result = $query->execute();

    $entity = NULL;
    if (count($result) > 0) {
      $entity = entity_load_single('erpal_reports', key($result ['erpal_reports']));
      $entity_wrapper = entity_metadata_wrapper('erpal_reports', $entity);

      if ($options['no_cache'] == FALSE && $entity_wrapper->dirty->value() == 0) {
        $ret = (array)json_decode($entity_wrapper->data->value());        
        $ret['report_id'] = $entity->id;        
        return $ret;         
      }
    }

    $retval = self::create($options);

    if ($entity == NULL) {
      $entity = entity_create('erpal_reports', array('type' =>'erpal_project_timetracking_reports'));
    }

    // store data to entity, reset dirty flag
    $entity->data = drupal_json_encode($retval);
    $entity->dirty = 0;
    $entity->field_erp_prj_tt_rep_dim_prj = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_project'] === TRUE ? 1 : 0)));
    $entity->field_erp_prj_tt_rep_dim_cust = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_customer'] === TRUE ? 1 : 0)));
    $entity->field_erp_prj_tt_rep_dim_cuscat = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_customercategory'] === TRUE ? 1 : 0)));
    $entity->field_erp_prj_tt_rep_dim_user = array(LANGUAGE_NONE => array(0 => array('value' => $options['dim_user'] === TRUE ? 1 : 0)));

    if (count($options['customers']) > 0) {
      $customers = array();
      foreach($options['customers'] as $customer) {
        $customers[] = array('target_id' => $customer);
      }
      $entity->field_customer_ref = array(LANGUAGE_NONE => $customers);
    }

    if (count($options['customer_categories']) > 0) {
      $customer_categories = array();
      foreach($options['customer_categories'] as $customer_category) {
        $customer_categories[] = array('value' => $customer_category);
      }

      $entity->field_customer_category = array(LANGUAGE_NONE => $customer_categories);
    }

    if (count($options['task_categories']) > 0) {
      $service_categories = array();
      foreach($options['task_categories'] as $service_category) {
        $service_categories[] = array('tid' => $service_category);
      }
      $entity->field_task_category_term = array(LANGUAGE_NONE => $service_categories);
    }

    $entity->save();

    $retval['report_id'] = $entity->id;

    //save details first and remove this big data
    _erpal_reports_save_details($entity, $retval, TRUE);
    //$entity->data = json_encode($retval);

    return $retval;
  }
}
